package com.code.aaron.micstream;

import java.lang.Math;
import java.util.ArrayList;

import android.annotation.TargetApi;
import android.media.AudioFormat;
import android.media.AudioRecord;
import android.media.MediaRecorder;

import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.PluginRegistry.Registrar;

/** MicStreamPlugin
 *  In reference to flutters official sensors plugin
 *  and the example of the streams_channel (v0.2.2) plugin
 */

@TargetApi(16)  // Should be unnecessary, but isn't // fix build.gradle...?
public class MicStreamPlugin implements EventChannel.StreamHandler {
    private static final String MICROPHONE_CHANNEL_NAME = "aaron.code.com/mic_stream";

    /**
     * Plugin registration
     */
    public static void registerWith(Registrar registrar) {
        final EventChannel microphone = new EventChannel(registrar.messenger(), MICROPHONE_CHANNEL_NAME);
        microphone.setStreamHandler(new MicStreamPlugin());
    }

    private EventChannel.EventSink eventSink;

    // Audio recorder + initial values
    private static volatile AudioRecord recorder;

    private int AUDIO_SOURCE = MediaRecorder.AudioSource.DEFAULT;
    private int SAMPLE_RATE = 16000;
    private int CHANNEL_CONFIG = AudioFormat.CHANNEL_IN_MONO;
    private int AUDIO_FORMAT = AudioFormat.ENCODING_PCM_8BIT;
    private int BUFFER_SIZE = AudioRecord.getMinBufferSize(SAMPLE_RATE, CHANNEL_CONFIG, AUDIO_FORMAT);

    // Runnable management
    private volatile boolean record = false;
    private volatile boolean isRecording = false;

    private final Runnable runnable = new Runnable() {
        @Override
        public void run() {
            isRecording = true;

            // Repeatedly push audio samples to stream
            while (record) {

                // 8 Bit encoding
                if (AUDIO_FORMAT == AudioFormat.ENCODING_PCM_8BIT) {

                    // Read audio data into new byte array
                    byte[] data = new byte[BUFFER_SIZE];
                    recorder.read(data, 0, BUFFER_SIZE);

                    // push data into stream
                    try {
                        eventSink.success(data);
                    } catch (IllegalArgumentException e) {
                        System.out.println("mic_stream: " + data.toString() + " is not valid!");
                        break;
                    }
                }

                else if (AUDIO_FORMAT == AudioFormat.ENCODING_PCM_16BIT) {

                    // Read audio data into new short array
                    short[] data_s = new short[BUFFER_SIZE];
                    byte[] data_b = new byte[BUFFER_SIZE * 2];
                    recorder.read(data_s, 0, BUFFER_SIZE);

                    // Split short into two bytes
                    for (int i = 0; i < BUFFER_SIZE; i++) {
                        //data_s[i] += 32768;
                        data_b[2 * i] = (byte) Math.floor(data_s[i] / 256.0);
                        data_b[2*i+1] = (byte) (data_s[i] % 256);
                    }

                    // push data into stream
                    try {
                        eventSink.success(data_b);
                    } catch (IllegalArgumentException e) {
                        System.out.println("mic_stream: " + data_b + " is not valid!");
                        eventSink.error("-2", "Invalid Data", e);
                    }
                }
                else {
                    eventSink.error("-3", "Invalid Audio Format specified", null);
                    break;
                }
            }
            isRecording = false;
        }
    };

    @Override
    public void onListen(Object args, final EventChannel.EventSink eventSink) {
        if (isRecording) return;

        ArrayList<Integer> config = (ArrayList<Integer>) args;

        // Set parameters, if available
        switch(config.size()) {
            case 4:
                AUDIO_FORMAT = config.get(3);
            case 3:
                CHANNEL_CONFIG = config.get(2);
            case 2:
                SAMPLE_RATE = config.get(1);
            case 1:
                AUDIO_SOURCE = config.get(0);
            default:
                try {
                    BUFFER_SIZE = AudioRecord.getMinBufferSize(SAMPLE_RATE, CHANNEL_CONFIG, AUDIO_FORMAT);
                } catch (Exception e) {
                    eventSink.error("-3", "Invalid AudioRecord parameters", e);
                }
        }

        this.eventSink = eventSink;

        // Try to initialize and start the recorder
        recorder = new AudioRecord(AUDIO_SOURCE, SAMPLE_RATE, CHANNEL_CONFIG, AUDIO_FORMAT, BUFFER_SIZE);
        if (recorder.getState() != AudioRecord.STATE_INITIALIZED) eventSink.error("-1", "PlatformError", null);
        recorder.startRecording();

        // Start runnable
        record = true;
        new Thread(runnable).start();
    }

    @Override
    public void onCancel(Object o) {
        // Stop runnable
        record = false;

        // Stop and reset audio recorder
        recorder.stop();
        recorder.release();
        recorder = null;
    }
}